%
% function Corr = 
%       corrFactor( Rmir, Rin, win, x, y, z, 
%                   n[-1], L[0], options[0], err[1e-5], k[2pi/1.064e-6] )
% field at distance z from a lens/mirror = HG00(x,y,z) * corrFactor(x,y,z)
% 
% parameters up to z are necessary to be specified, rest is optional.
%
% 1) n is defined and positive, this calculates the transmission through a lens with
% macroscopic thickness of L, otherwise, this calculates a reflection by a
% mirror.
% 2) Rmir is the optic surface curvature. For a lens, this is the side
% outgoing field is leaving.
% 3) Rin and win are the curvature and width of the field going to optic
% 4) (x,y,z) are the locations of the out going fields to be calculated. 
% x can be an array, y is not supported (it is taken to be 0), z is scalar.
% 5) options specify 
%       a) if the optic surface is parabolic or spherical, and
%             options = 0, 1 or -1 for spherical and 
%             options = 2 or -2 for parabolic
%       b) which method of integration to be used. When optimal method
%          fails to converge, try slow method
%             options = 0, 1 or 2 for optimal method
%             options = -1 or -2 for slow method
% 6) err is the relative error of the answer
% 7) k = 2 pi / lambda, default is 1.064e-6.
%
function Corr = corrFactor( Rmir, Rin, win, x, y, z, n, L, options, err, k )

if ~exist( 'options', 'var' )
    options = 0;
end

if options >= 0
    intMethod = 'auto';
else
    intMethod = 'iterated';
end
parab = abs(options) == 2;

% Rmir is finite for spherical mirror
% for a reflector, n = -1, which makes -(n-1) d = 2 d
if ~exist('n', 'var') || n <= 0
    n = -1;
end

if ~exist('err', 'var')
    err = 1e-5;
end
if ~exist( 'k', 'var' )
    k = 2*pi/1.064e-6;
end

if n > 0 && exist('L', 'var') && (L > 0)
    [Rin, win] = RWSubs( Rin, win, n, L, k );
end
Ropt = 1/((n-1)/Rmir + 1/Rin);

A = 1/win^2 + 1i*k/2*(1/Ropt + 1/z);
B = Ropt / ( z + Ropt - 2i*z*Ropt/(k*win^2) );

Corr = zeros(size(x));

lowerLimit = min(exp(-2*4^2), err);
for ix = 1:length(x)
    Corr(ix) = integral2( @(u, p) ...
        corrFunc( sqrt(-log(u)/A).*cos(p) + B*x(ix), sqrt(-log(u)/A).*sin(p), x(ix), z, Rmir, parab, n, k ), ...
        lowerLimit, 1, -pi, pi, 'RelTol', err, 'method', intMethod );
end

return

function val = corrFunc( u, v, x, z, Rmir, parab, n, k )

rsq = u.^2 + v.^2;
musq = (x-u).^2 + v.^2;

% delRmir is surface shape, del = delRmir - r^2/2 Rmir
if parab
    delRmir = rsq/(2*Rmir);
    del = zeros( size(rsq) );
    slopeIn = sqrt(1-rsq/Rmir^2);
else
    delRmir = rsq ./ (Rmir*(1+sqrt(1-rsq/Rmir^2)));
    del = delRmir.^2 / (2*Rmir);
    slopeIn = sqrt( (Rmir^2-2*rsq) ./ (Rmir^2-rsq) );
end

zdel = z - delRmir;
rho2 = musq + zdel.^2;
rho = sqrt(rho2);

drz = musq.*(4*z*delRmir - musq)./(2*z*(rho+zdel).*(rho+z+delRmir));

% n = -1 for reflection, which makes -(n-1)del = 2 del
phi = k*(-(n-1)*del - drz);

if max(max(imag(phi))) < -10
    ind = find( abs(phi) == max(max(abs(phi))) );
    fprintf('%d: max phi > 1 : %g + 1i*%g\n', ind, real(phi(ind)), imag(phi(ind)) );
end

val = ( slopeIn * z .* zdel ./ rho2 .* exp( 1i*phi ) .* ( 1 - 1i./(k*rho) ) ) / (2*pi);
% val = ( z ./ rho .* exp( 1i*phi ) .* ( 1 - 1i./(k*rho) ) ) / (2*pi);

% val =  exp( 1i*phi ) - 1;
% zCorr = (delRmir.*zdel-musq) ./ rho2;  % z*zdel/rho2 - 1 
% val = ( val.*zCorr + val + zCorr ) / (2*pi);

return